import json

ALPACA_VAULT_CONFIG_ABI = json.loads('''
[
    {
        "inputs": [
            {
                "internalType": "address",
                "name": "worker",
                "type": "address"
            }
        ],
        "name": "acceptDebt",
        "outputs": [
            {
                "internalType": "bool",
                "name": "",
                "type": "bool"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "address",
                "name": "addStrats",
                "type": "address"
            }
        ],
        "name": "approvedAddStrategies",
        "outputs": [
            {
                "internalType": "bool",
                "name": "",
                "type": "bool"
            }
        ],
        "stateMutability": "nonpayable",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "getFairLaunchAddr",
        "outputs": [
            {
                "internalType": "address",
                "name": "",
                "type": "address"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "uint256",
                "name": "debt",
                "type": "uint256"
            },
            {
                "internalType": "uint256",
                "name": "floating",
                "type": "uint256"
            }
        ],
        "name": "getInterestRate",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "getKillBps",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "getKillTreasuryBps",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "getReservePoolBps",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "getTreasuryAddr",
        "outputs": [
            {
                "internalType": "address",
                "name": "",
                "type": "address"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "getWNativeRelayer",
        "outputs": [
            {
                "internalType": "address",
                "name": "",
                "type": "address"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "getWrappedNativeAddr",
        "outputs": [
            {
                "internalType": "address",
                "name": "",
                "type": "address"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "address",
                "name": "worker",
                "type": "address"
            }
        ],
        "name": "isWorker",
        "outputs": [
            {
                "internalType": "bool",
                "name": "",
                "type": "bool"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "address",
                "name": "worker",
                "type": "address"
            }
        ],
        "name": "isWorkerReserveConsistent",
        "outputs": [
            {
                "internalType": "bool",
                "name": "",
                "type": "bool"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "address",
                "name": "worker",
                "type": "address"
            }
        ],
        "name": "isWorkerStable",
        "outputs": [
            {
                "internalType": "bool",
                "name": "",
                "type": "bool"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "address",
                "name": "worker",
                "type": "address"
            },
            {
                "internalType": "uint256",
                "name": "debt",
                "type": "uint256"
            }
        ],
        "name": "killFactor",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "address",
                "name": "worker",
                "type": "address"
            },
            {
                "internalType": "uint256",
                "name": "debt",
                "type": "uint256"
            },
            {
                "internalType": "address",
                "name": "positionOwner",
                "type": "address"
            }
        ],
        "name": "killFactor",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "minDebtSize",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "address",
                "name": "worker",
                "type": "address"
            },
            {
                "internalType": "uint256",
                "name": "debt",
                "type": "uint256"
            },
            {
                "internalType": "address",
                "name": "positionOwner",
                "type": "address"
            }
        ],
        "name": "rawKillFactor",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "address",
                "name": "worker",
                "type": "address"
            },
            {
                "internalType": "uint256",
                "name": "debt",
                "type": "uint256"
            }
        ],
        "name": "rawKillFactor",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "address",
                "name": "caller",
                "type": "address"
            }
        ],
        "name": "whitelistedCallers",
        "outputs": [
            {
                "internalType": "bool",
                "name": "",
                "type": "bool"
            }
        ],
        "stateMutability": "nonpayable",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "address",
                "name": "caller",
                "type": "address"
            }
        ],
        "name": "whitelistedLiquidators",
        "outputs": [
            {
                "internalType": "bool",
                "name": "",
                "type": "bool"
            }
        ],
        "stateMutability": "nonpayable",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "address",
                "name": "worker",
                "type": "address"
            },
            {
                "internalType": "uint256",
                "name": "debt",
                "type": "uint256"
            },
            {
                "internalType": "address",
                "name": "positionOwner",
                "type": "address"
            }
        ],
        "name": "workFactor",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "address",
                "name": "worker",
                "type": "address"
            },
            {
                "internalType": "uint256",
                "name": "debt",
                "type": "uint256"
            }
        ],
        "name": "workFactor",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    }
]
''')




import json

STDCOMPTROLLER = json.loads("""
[
  {
    "constant": true,
    "inputs": [],
    "name": "isComptroller",
    "outputs": [
      {
        "name": "",
        "type": "bool"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0x007e3dd2"
  },
  {
    "constant": false,
    "inputs": [
      {
        "name": "cToken",
        "type": "address"
      },
      {
        "name": "payer",
        "type": "address"
      },
      {
        "name": "borrower",
        "type": "address"
      },
      {
        "name": "repayAmount",
        "type": "uint256"
      },
      {
        "name": "borrowerIndex",
        "type": "uint256"
      }
    ],
    "name": "repayBorrowVerify",
    "outputs": [],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0x1ededc91"
  },
  {
    "constant": false,
    "inputs": [
      {
        "name": "cToken",
        "type": "address"
      },
      {
        "name": "payer",
        "type": "address"
      },
      {
        "name": "borrower",
        "type": "address"
      },
      {
        "name": "repayAmount",
        "type": "uint256"
      }
    ],
    "name": "repayBorrowAllowed",
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0x24008a62"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "pendingAdmin",
    "outputs": [
      {
        "name": "",
        "type": "address"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0x26782247"
  },
  {
    "constant": false,
    "inputs": [
      {
        "name": "newCloseFactorMantissa",
        "type": "uint256"
      }
    ],
    "name": "_setCloseFactor",
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0x317b0b77"
  },
  {
    "constant": false,
    "inputs": [
      {
        "name": "unitroller",
        "type": "address"
      },
      {
        "name": "_oracle",
        "type": "address"
      },
      {
        "name": "_closeFactorMantissa",
        "type": "uint256"
      },
      {
        "name": "_maxAssets",
        "type": "uint256"
      },
      {
        "name": "reinitializing",
        "type": "bool"
      }
    ],
    "name": "_become",
    "outputs": [],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0x32000e00"
  },
  {
    "constant": false,
    "inputs": [
      {
        "name": "cToken",
        "type": "address"
      },
      {
        "name": "minter",
        "type": "address"
      },
      {
        "name": "mintAmount",
        "type": "uint256"
      },
      {
        "name": "mintTokens",
        "type": "uint256"
      }
    ],
    "name": "mintVerify",
    "outputs": [],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0x41c728b9"
  },
  {
    "constant": false,
    "inputs": [
      {
        "name": "cTokenBorrowed",
        "type": "address"
      },
      {
        "name": "cTokenCollateral",
        "type": "address"
      },
      {
        "name": "liquidator",
        "type": "address"
      },
      {
        "name": "borrower",
        "type": "address"
      },
      {
        "name": "repayAmount",
        "type": "uint256"
      },
      {
        "name": "seizeTokens",
        "type": "uint256"
      }
    ],
    "name": "liquidateBorrowVerify",
    "outputs": [],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0x47ef3b3b"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "liquidationIncentiveMantissa",
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0x4ada90af"
  },
  {
    "constant": false,
    "inputs": [
      {
        "name": "cToken",
        "type": "address"
      },
      {
        "name": "minter",
        "type": "address"
      },
      {
        "name": "mintAmount",
        "type": "uint256"
      }
    ],
    "name": "mintAllowed",
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0x4ef4c3e1"
  },
  {
    "constant": false,
    "inputs": [
      {
        "name": "newLiquidationIncentiveMantissa",
        "type": "uint256"
      }
    ],
    "name": "_setLiquidationIncentive",
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0x4fd42e17"
  },
  {
    "constant": false,
    "inputs": [
      {
        "name": "cToken",
        "type": "address"
      },
      {
        "name": "redeemer",
        "type": "address"
      },
      {
        "name": "redeemAmount",
        "type": "uint256"
      },
      {
        "name": "redeemTokens",
        "type": "uint256"
      }
    ],
    "name": "redeemVerify",
    "outputs": [],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0x51dff989"
  },
  {
    "constant": false,
    "inputs": [
      {
        "name": "newOracle",
        "type": "address"
      }
    ],
    "name": "_setPriceOracle",
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0x55ee1fe1"
  },
  {
    "constant": false,
    "inputs": [
      {
        "name": "cToken",
        "type": "address"
      },
      {
        "name": "borrower",
        "type": "address"
      },
      {
        "name": "borrowAmount",
        "type": "uint256"
      }
    ],
    "name": "borrowVerify",
    "outputs": [],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0x5c778605"
  },
  {
    "constant": true,
    "inputs": [
      {
        "name": "account",
        "type": "address"
      }
    ],
    "name": "getAccountLiquidity",
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      },
      {
        "name": "",
        "type": "uint256"
      },
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0x5ec88c79"
  },
  {
    "constant": false,
    "inputs": [
      {
        "name": "cTokenBorrowed",
        "type": "address"
      },
      {
        "name": "cTokenCollateral",
        "type": "address"
      },
      {
        "name": "liquidator",
        "type": "address"
      },
      {
        "name": "borrower",
        "type": "address"
      },
      {
        "name": "repayAmount",
        "type": "uint256"
      }
    ],
    "name": "liquidateBorrowAllowed",
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0x5fc7e71e"
  },
  {
    "constant": false,
    "inputs": [
      {
        "name": "cToken",
        "type": "address"
      },
      {
        "name": "src",
        "type": "address"
      },
      {
        "name": "dst",
        "type": "address"
      },
      {
        "name": "transferTokens",
        "type": "uint256"
      }
    ],
    "name": "transferVerify",
    "outputs": [],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0x6a56947e"
  },
  {
    "constant": false,
    "inputs": [
      {
        "name": "cTokenCollateral",
        "type": "address"
      },
      {
        "name": "cTokenBorrowed",
        "type": "address"
      },
      {
        "name": "liquidator",
        "type": "address"
      },
      {
        "name": "borrower",
        "type": "address"
      },
      {
        "name": "seizeTokens",
        "type": "uint256"
      }
    ],
    "name": "seizeVerify",
    "outputs": [],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0x6d35bf91"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "oracle",
    "outputs": [
      {
        "name": "",
        "type": "address"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0x7dc0d1d0"
  },
  {
    "constant": true,
    "inputs": [
      {
        "name": "",
        "type": "address"
      }
    ],
    "name": "markets",
    "outputs": [
      {
        "name": "isListed",
        "type": "bool"
      },
      {
        "name": "collateralFactorMantissa",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0x8e8f294b"
  },
  {
    "constant": true,
    "inputs": [
      {
        "name": "account",
        "type": "address"
      },
      {
        "name": "cToken",
        "type": "address"
      }
    ],
    "name": "checkMembership",
    "outputs": [
      {
        "name": "",
        "type": "bool"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0x929fe9a1"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "maxAssets",
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0x94b2294b"
  },
  {
    "constant": false,
    "inputs": [
      {
        "name": "cToken",
        "type": "address"
      }
    ],
    "name": "_supportMarket",
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0xa76b3fda"
  },
  {
    "constant": true,
    "inputs": [
      {
        "name": "account",
        "type": "address"
      }
    ],
    "name": "getAssetsIn",
    "outputs": [
      {
        "name": "",
        "type": "address[]"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0xabfceffc"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "comptrollerImplementation",
    "outputs": [
      {
        "name": "",
        "type": "address"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0xbb82aa5e"
  },
  {
    "constant": false,
    "inputs": [
      {
        "name": "cToken",
        "type": "address"
      },
      {
        "name": "src",
        "type": "address"
      },
      {
        "name": "dst",
        "type": "address"
      },
      {
        "name": "transferTokens",
        "type": "uint256"
      }
    ],
    "name": "transferAllowed",
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0xbdcdc258"
  },
  {
    "constant": false,
    "inputs": [
      {
        "name": "cTokens",
        "type": "address[]"
      }
    ],
    "name": "enterMarkets",
    "outputs": [
      {
        "name": "",
        "type": "uint256[]"
      }
    ],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0xc2998238"
  },
  {
    "constant": true,
    "inputs": [
      {
        "name": "cTokenBorrowed",
        "type": "address"
      },
      {
        "name": "cTokenCollateral",
        "type": "address"
      },
      {
        "name": "repayAmount",
        "type": "uint256"
      }
    ],
    "name": "liquidateCalculateSeizeTokens",
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      },
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0xc488847b"
  },
  {
    "constant": false,
    "inputs": [
      {
        "name": "cTokenCollateral",
        "type": "address"
      },
      {
        "name": "cTokenBorrowed",
        "type": "address"
      },
      {
        "name": "liquidator",
        "type": "address"
      },
      {
        "name": "borrower",
        "type": "address"
      },
      {
        "name": "seizeTokens",
        "type": "uint256"
      }
    ],
    "name": "seizeAllowed",
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0xd02f7351"
  },
  {
    "constant": false,
    "inputs": [
      {
        "name": "newMaxAssets",
        "type": "uint256"
      }
    ],
    "name": "_setMaxAssets",
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0xd9226ced"
  },
  {
    "constant": false,
    "inputs": [
      {
        "name": "cToken",
        "type": "address"
      },
      {
        "name": "borrower",
        "type": "address"
      },
      {
        "name": "borrowAmount",
        "type": "uint256"
      }
    ],
    "name": "borrowAllowed",
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0xda3d454c"
  },
  {
    "constant": true,
    "inputs": [
      {
        "name": "",
        "type": "address"
      },
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "name": "accountAssets",
    "outputs": [
      {
        "name": "",
        "type": "address"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0xdce15449"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "pendingComptrollerImplementation",
    "outputs": [
      {
        "name": "",
        "type": "address"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0xdcfbc0c7"
  },
  {
    "constant": false,
    "inputs": [
      {
        "name": "cToken",
        "type": "address"
      },
      {
        "name": "newCollateralFactorMantissa",
        "type": "uint256"
      }
    ],
    "name": "_setCollateralFactor",
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0xe4028eee"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "closeFactorMantissa",
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0xe8755446"
  },
  {
    "constant": false,
    "inputs": [
      {
        "name": "cToken",
        "type": "address"
      },
      {
        "name": "redeemer",
        "type": "address"
      },
      {
        "name": "redeemTokens",
        "type": "uint256"
      }
    ],
    "name": "redeemAllowed",
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0xeabe7d91"
  },
  {
    "constant": false,
    "inputs": [
      {
        "name": "cTokenAddress",
        "type": "address"
      }
    ],
    "name": "exitMarket",
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function",
    "signature": "0xede4edd0"
  },
  {
    "constant": true,
    "inputs": [],
    "name": "admin",
    "outputs": [
      {
        "name": "",
        "type": "address"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function",
    "signature": "0xf851a440"
  },
  {
    "inputs": [],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "constructor",
    "signature": "constructor"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "name": "cToken",
        "type": "address"
      }
    ],
    "name": "MarketListed",
    "type": "event",
    "signature": "0xcf583bb0c569eb967f806b11601c4cb93c10310485c67add5f8362c2f212321f"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "name": "cToken",
        "type": "address"
      },
      {
        "indexed": false,
        "name": "account",
        "type": "address"
      }
    ],
    "name": "MarketEntered",
    "type": "event",
    "signature": "0x3ab23ab0d51cccc0c3085aec51f99228625aa1a922b3a8ca89a26b0f2027a1a5"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "name": "cToken",
        "type": "address"
      },
      {
        "indexed": false,
        "name": "account",
        "type": "address"
      }
    ],
    "name": "MarketExited",
    "type": "event",
    "signature": "0xe699a64c18b07ac5b7301aa273f36a2287239eb9501d81950672794afba29a0d"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "name": "oldCloseFactorMantissa",
        "type": "uint256"
      },
      {
        "indexed": false,
        "name": "newCloseFactorMantissa",
        "type": "uint256"
      }
    ],
    "name": "NewCloseFactor",
    "type": "event",
    "signature": "0x3b9670cf975d26958e754b57098eaa2ac914d8d2a31b83257997b9f346110fd9"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "name": "cToken",
        "type": "address"
      },
      {
        "indexed": false,
        "name": "oldCollateralFactorMantissa",
        "type": "uint256"
      },
      {
        "indexed": false,
        "name": "newCollateralFactorMantissa",
        "type": "uint256"
      }
    ],
    "name": "NewCollateralFactor",
    "type": "event",
    "signature": "0x70483e6592cd5182d45ac970e05bc62cdcc90e9d8ef2c2dbe686cf383bcd7fc5"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "name": "oldLiquidationIncentiveMantissa",
        "type": "uint256"
      },
      {
        "indexed": false,
        "name": "newLiquidationIncentiveMantissa",
        "type": "uint256"
      }
    ],
    "name": "NewLiquidationIncentive",
    "type": "event",
    "signature": "0xaeba5a6c40a8ac138134bff1aaa65debf25971188a58804bad717f82f0ec1316"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "name": "oldMaxAssets",
        "type": "uint256"
      },
      {
        "indexed": false,
        "name": "newMaxAssets",
        "type": "uint256"
      }
    ],
    "name": "NewMaxAssets",
    "type": "event",
    "signature": "0x7093cf1eb653f749c3ff531d6df7f92764536a7fa0d13530cd26e070780c32ea"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "name": "oldPriceOracle",
        "type": "address"
      },
      {
        "indexed": false,
        "name": "newPriceOracle",
        "type": "address"
      }
    ],
    "name": "NewPriceOracle",
    "type": "event",
    "signature": "0xd52b2b9b7e9ee655fcb95d2e5b9e0c9f69e7ef2b8e9d2d0ea78402d576d22e22"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "name": "error",
        "type": "uint256"
      },
      {
        "indexed": false,
        "name": "info",
        "type": "uint256"
      },
      {
        "indexed": false,
        "name": "detail",
        "type": "uint256"
      }
    ],
    "name": "Failure",
    "type": "event",
    "signature": "0x45b96fe442630264581b197e84bbada861235052c5a1aadfff9ea4e40a969aa0"
  }
]
""")
        


class Query:
    address = "address"
    abi = "abi"
    function = "function"
    params = "params"
    block_number = "block_number"
    module = "module"

    # query types
    lp_token_list = 'lp_token_list'
    farming_lp_token_list = 'farming_lp_token_list'
    important_lp_token_list = 'important_lp_token_list'
    dex_user_token_balance= 'dex_user_token_balance'
    lp_token_info = 'lp_token_info'
    dex_user_info = 'dex_user_info'
    dex_user_nft = 'dex_user_nft'
    token_pair_balance = 'token_pair_balance'
    token_balance = 'token_balance'
    nft_balance = 'nft_balance'
    deposit_borrow = 'deposit_borrow'
    staking_reward = 'staking_reward'
    protocol_reward = 'protocol_reward'
    protocol_apy = 'protocol_apy'
    health_factor = 'health_factor'
    deposit_borrow_health_factor = 'deposit_borrow_health_factor'
    protocol_supply_borrow = 'protocol_supply_borrow'
    all = [token_balance, nft_balance, deposit_borrow, protocol_reward, protocol_apy, protocol_supply_borrow]
    balance = [nft_balance, token_balance]
    # entity_id
    token = "token"



import json

LIDO_LIQUIDITY_FARMING_POOL_ABI = json.loads('''
[
  {
    "name": "TokenExchange",
    "inputs": [
      {
        "type": "address",
        "name": "buyer",
        "indexed": true
      },
      {
        "type": "int128",
        "name": "sold_id",
        "indexed": false
      },
      {
        "type": "uint256",
        "name": "tokens_sold",
        "indexed": false
      },
      {
        "type": "int128",
        "name": "bought_id",
        "indexed": false
      },
      {
        "type": "uint256",
        "name": "tokens_bought",
        "indexed": false
      }
    ],
    "anonymous": false,
    "type": "event"
  },
  {
    "name": "TokenExchangeUnderlying",
    "inputs": [
      {
        "type": "address",
        "name": "buyer",
        "indexed": true
      },
      {
        "type": "int128",
        "name": "sold_id",
        "indexed": false
      },
      {
        "type": "uint256",
        "name": "tokens_sold",
        "indexed": false
      },
      {
        "type": "int128",
        "name": "bought_id",
        "indexed": false
      },
      {
        "type": "uint256",
        "name": "tokens_bought",
        "indexed": false
      }
    ],
    "anonymous": false,
    "type": "event"
  },
  {
    "name": "AddLiquidity",
    "inputs": [
      {
        "type": "address",
        "name": "provider",
        "indexed": true
      },
      {
        "type": "uint256[2]",
        "name": "token_amounts",
        "indexed": false
      },
      {
        "type": "uint256[2]",
        "name": "fees",
        "indexed": false
      },
      {
        "type": "uint256",
        "name": "invariant",
        "indexed": false
      },
      {
        "type": "uint256",
        "name": "token_supply",
        "indexed": false
      }
    ],
    "anonymous": false,
    "type": "event"
  },
  {
    "name": "RemoveLiquidity",
    "inputs": [
      {
        "type": "address",
        "name": "provider",
        "indexed": true
      },
      {
        "type": "uint256[2]",
        "name": "token_amounts",
        "indexed": false
      },
      {
        "type": "uint256[2]",
        "name": "fees",
        "indexed": false
      },
      {
        "type": "uint256",
        "name": "token_supply",
        "indexed": false
      }
    ],
    "anonymous": false,
    "type": "event"
  },
  {
    "name": "RemoveLiquidityOne",
    "inputs": [
      {
        "type": "address",
        "name": "provider",
        "indexed": true
      },
      {
        "type": "uint256",
        "name": "token_amount",
        "indexed": false
      },
      {
        "type": "uint256",
        "name": "coin_amount",
        "indexed": false
      }
    ],
    "anonymous": false,
    "type": "event"
  },
  {
    "name": "RemoveLiquidityImbalance",
    "inputs": [
      {
        "type": "address",
        "name": "provider",
        "indexed": true
      },
      {
        "type": "uint256[2]",
        "name": "token_amounts",
        "indexed": false
      },
      {
        "type": "uint256[2]",
        "name": "fees",
        "indexed": false
      },
      {
        "type": "uint256",
        "name": "invariant",
        "indexed": false
      },
      {
        "type": "uint256",
        "name": "token_supply",
        "indexed": false
      }
    ],
    "anonymous": false,
    "type": "event"
  },
  {
    "name": "CommitNewAdmin",
    "inputs": [
      {
        "type": "uint256",
        "name": "deadline",
        "indexed": true
      },
      {
        "type": "address",
        "name": "admin",
        "indexed": true
      }
    ],
    "anonymous": false,
    "type": "event"
  },
  {
    "name": "NewAdmin",
    "inputs": [
      {
        "type": "address",
        "name": "admin",
        "indexed": true
      }
    ],
    "anonymous": false,
    "type": "event"
  },
  {
    "name": "CommitNewFee",
    "inputs": [
      {
        "type": "uint256",
        "name": "deadline",
        "indexed": true
      },
      {
        "type": "uint256",
        "name": "fee",
        "indexed": false
      },
      {
        "type": "uint256",
        "name": "admin_fee",
        "indexed": false
      }
    ],
    "anonymous": false,
    "type": "event"
  },
  {
    "name": "NewFee",
    "inputs": [
      {
        "type": "uint256",
        "name": "fee",
        "indexed": false
      },
      {
        "type": "uint256",
        "name": "admin_fee",
        "indexed": false
      }
    ],
    "anonymous": false,
    "type": "event"
  },
  {
    "name": "RampA",
    "inputs": [
      {
        "type": "uint256",
        "name": "old_A",
        "indexed": false
      },
      {
        "type": "uint256",
        "name": "new_A",
        "indexed": false
      },
      {
        "type": "uint256",
        "name": "initial_time",
        "indexed": false
      },
      {
        "type": "uint256",
        "name": "future_time",
        "indexed": false
      }
    ],
    "anonymous": false,
    "type": "event"
  },
  {
    "name": "StopRampA",
    "inputs": [
      {
        "type": "uint256",
        "name": "A",
        "indexed": false
      },
      {
        "type": "uint256",
        "name": "t",
        "indexed": false
      }
    ],
    "anonymous": false,
    "type": "event"
  },
  {
    "outputs": [],
    "inputs": [
      {
        "type": "address",
        "name": "_owner"
      },
      {
        "type": "address[2]",
        "name": "_coins"
      },
      {
        "type": "address",
        "name": "_pool_token"
      },
      {
        "type": "uint256",
        "name": "_A"
      },
      {
        "type": "uint256",
        "name": "_fee"
      },
      {
        "type": "uint256",
        "name": "_admin_fee"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "constructor"
  },
  {
    "name": "A",
    "outputs": [
      {
        "type": "uint256",
        "name": ""
      }
    ],
    "inputs": [],
    "stateMutability": "view",
    "type": "function",
    "gas": 5289
  },
  {
    "name": "A_precise",
    "outputs": [
      {
        "type": "uint256",
        "name": ""
      }
    ],
    "inputs": [],
    "stateMutability": "view",
    "type": "function",
    "gas": 5251
  },
  {
    "name": "balances",
    "outputs": [
      {
        "type": "uint256",
        "name": ""
      }
    ],
    "inputs": [
      {
        "type": "uint256",
        "name": "i"
      }
    ],
    "stateMutability": "view",
    "type": "function",
    "gas": 5076
  },
  {
    "name": "get_virtual_price",
    "outputs": [
      {
        "type": "uint256",
        "name": ""
      }
    ],
    "inputs": [],
    "stateMutability": "view",
    "type": "function",
    "gas": 1114301
  },
  {
    "name": "calc_token_amount",
    "outputs": [
      {
        "type": "uint256",
        "name": ""
      }
    ],
    "inputs": [
      {
        "type": "uint256[2]",
        "name": "amounts"
      },
      {
        "type": "bool",
        "name": "is_deposit"
      }
    ],
    "stateMutability": "view",
    "type": "function",
    "gas": 2218181
  },
  {
    "name": "add_liquidity",
    "outputs": [
      {
        "type": "uint256",
        "name": ""
      }
    ],
    "inputs": [
      {
        "type": "uint256[2]",
        "name": "amounts"
      },
      {
        "type": "uint256",
        "name": "min_mint_amount"
      }
    ],
    "stateMutability": "payable",
    "type": "function",
    "gas": 3484118
  },
  {
    "name": "get_dy",
    "outputs": [
      {
        "type": "uint256",
        "name": ""
      }
    ],
    "inputs": [
      {
        "type": "int128",
        "name": "i"
      },
      {
        "type": "int128",
        "name": "j"
      },
      {
        "type": "uint256",
        "name": "dx"
      }
    ],
    "stateMutability": "view",
    "type": "function",
    "gas": 2654541
  },
  {
    "name": "exchange",
    "outputs": [
      {
        "type": "uint256",
        "name": ""
      }
    ],
    "inputs": [
      {
        "type": "int128",
        "name": "i"
      },
      {
        "type": "int128",
        "name": "j"
      },
      {
        "type": "uint256",
        "name": "dx"
      },
      {
        "type": "uint256",
        "name": "min_dy"
      }
    ],
    "stateMutability": "payable",
    "type": "function",
    "gas": 2810134
  },
  {
    "name": "remove_liquidity",
    "outputs": [
      {
        "type": "uint256[2]",
        "name": ""
      }
    ],
    "inputs": [
      {
        "type": "uint256",
        "name": "_amount"
      },
      {
        "type": "uint256[2]",
        "name": "_min_amounts"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function",
    "gas": 160545
  },
  {
    "name": "remove_liquidity_imbalance",
    "outputs": [
      {
        "type": "uint256",
        "name": ""
      }
    ],
    "inputs": [
      {
        "type": "uint256[2]",
        "name": "_amounts"
      },
      {
        "type": "uint256",
        "name": "_max_burn_amount"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function",
    "gas": 3519382
  },
  {
    "name": "calc_withdraw_one_coin",
    "outputs": [
      {
        "type": "uint256",
        "name": ""
      }
    ],
    "inputs": [
      {
        "type": "uint256",
        "name": "_token_amount"
      },
      {
        "type": "int128",
        "name": "i"
      }
    ],
    "stateMutability": "view",
    "type": "function",
    "gas": 1435
  },
  {
    "name": "remove_liquidity_one_coin",
    "outputs": [
      {
        "type": "uint256",
        "name": ""
      }
    ],
    "inputs": [
      {
        "type": "uint256",
        "name": "_token_amount"
      },
      {
        "type": "int128",
        "name": "i"
      },
      {
        "type": "uint256",
        "name": "_min_amount"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function",
    "gas": 4113806
  },
  {
    "name": "ramp_A",
    "outputs": [],
    "inputs": [
      {
        "type": "uint256",
        "name": "_future_A"
      },
      {
        "type": "uint256",
        "name": "_future_time"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function",
    "gas": 151834
  },
  {
    "name": "stop_ramp_A",
    "outputs": [],
    "inputs": [],
    "stateMutability": "nonpayable",
    "type": "function",
    "gas": 148595
  },
  {
    "name": "commit_new_fee",
    "outputs": [],
    "inputs": [
      {
        "type": "uint256",
        "name": "new_fee"
      },
      {
        "type": "uint256",
        "name": "new_admin_fee"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function",
    "gas": 110431
  },
  {
    "name": "apply_new_fee",
    "outputs": [],
    "inputs": [],
    "stateMutability": "nonpayable",
    "type": "function",
    "gas": 153115
  },
  {
    "name": "revert_new_parameters",
    "outputs": [],
    "inputs": [],
    "stateMutability": "nonpayable",
    "type": "function",
    "gas": 21865
  },
  {
    "name": "commit_transfer_ownership",
    "outputs": [],
    "inputs": [
      {
        "type": "address",
        "name": "_owner"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "function",
    "gas": 74603
  },
  {
    "name": "apply_transfer_ownership",
    "outputs": [],
    "inputs": [],
    "stateMutability": "nonpayable",
    "type": "function",
    "gas": 116583
  },
  {
    "name": "revert_transfer_ownership",
    "outputs": [],
    "inputs": [],
    "stateMutability": "nonpayable",
    "type": "function",
    "gas": 21955
  },
  {
    "name": "withdraw_admin_fees",
    "outputs": [],
    "inputs": [],
    "stateMutability": "nonpayable",
    "type": "function",
    "gas": 137597
  },
  {
    "name": "donate_admin_fees",
    "outputs": [],
    "inputs": [],
    "stateMutability": "nonpayable",
    "type": "function",
    "gas": 42144
  },
  {
    "name": "kill_me",
    "outputs": [],
    "inputs": [],
    "stateMutability": "nonpayable",
    "type": "function",
    "gas": 37938
  },
  {
    "name": "unkill_me",
    "outputs": [],
    "inputs": [],
    "stateMutability": "nonpayable",
    "type": "function",
    "gas": 22075
  },
  {
    "name": "coins",
    "outputs": [
      {
        "type": "address",
        "name": ""
      }
    ],
    "inputs": [
      {
        "type": "uint256",
        "name": "arg0"
      }
    ],
    "stateMutability": "view",
    "type": "function",
    "gas": 2160
  },
  {
    "name": "admin_balances",
    "outputs": [
      {
        "type": "uint256",
        "name": ""
      }
    ],
    "inputs": [
      {
        "type": "uint256",
        "name": "arg0"
      }
    ],
    "stateMutability": "view",
    "type": "function",
    "gas": 2190
  },
  {
    "name": "fee",
    "outputs": [
      {
        "type": "uint256",
        "name": ""
      }
    ],
    "inputs": [],
    "stateMutability": "view",
    "type": "function",
    "gas": 2111
  },
  {
    "name": "admin_fee",
    "outputs": [
      {
        "type": "uint256",
        "name": ""
      }
    ],
    "inputs": [],
    "stateMutability": "view",
    "type": "function",
    "gas": 2141
  },
  {
    "name": "owner",
    "outputs": [
      {
        "type": "address",
        "name": ""
      }
    ],
    "inputs": [],
    "stateMutability": "view",
    "type": "function",
    "gas": 2171
  },
  {
    "name": "lp_token",
    "outputs": [
      {
        "type": "address",
        "name": ""
      }
    ],
    "inputs": [],
    "stateMutability": "view",
    "type": "function",
    "gas": 2201
  },
  {
    "name": "initial_A",
    "outputs": [
      {
        "type": "uint256",
        "name": ""
      }
    ],
    "inputs": [],
    "stateMutability": "view",
    "type": "function",
    "gas": 2231
  },
  {
    "name": "future_A",
    "outputs": [
      {
        "type": "uint256",
        "name": ""
      }
    ],
    "inputs": [],
    "stateMutability": "view",
    "type": "function",
    "gas": 2261
  },
  {
    "name": "initial_A_time",
    "outputs": [
      {
        "type": "uint256",
        "name": ""
      }
    ],
    "inputs": [],
    "stateMutability": "view",
    "type": "function",
    "gas": 2291
  },
  {
    "name": "future_A_time",
    "outputs": [
      {
        "type": "uint256",
        "name": ""
      }
    ],
    "inputs": [],
    "stateMutability": "view",
    "type": "function",
    "gas": 2321
  },
  {
    "name": "admin_actions_deadline",
    "outputs": [
      {
        "type": "uint256",
        "name": ""
      }
    ],
    "inputs": [],
    "stateMutability": "view",
    "type": "function",
    "gas": 2351
  },
  {
    "name": "transfer_ownership_deadline",
    "outputs": [
      {
        "type": "uint256",
        "name": ""
      }
    ],
    "inputs": [],
    "stateMutability": "view",
    "type": "function",
    "gas": 2381
  },
  {
    "name": "future_fee",
    "outputs": [
      {
        "type": "uint256",
        "name": ""
      }
    ],
    "inputs": [],
    "stateMutability": "view",
    "type": "function",
    "gas": 2411
  },
  {
    "name": "future_admin_fee",
    "outputs": [
      {
        "type": "uint256",
        "name": ""
      }
    ],
    "inputs": [],
    "stateMutability": "view",
    "type": "function",
    "gas": 2441
  },
  {
    "name": "future_owner",
    "outputs": [
      {
        "type": "address",
        "name": ""
      }
    ],
    "inputs": [],
    "stateMutability": "view",
    "type": "function",
    "gas": 2471
  }
]
''')


import json

TCV_ABI = json.loads('''
[
    {
      "inputs": [
        {
          "internalType": "contract IUniswapV3Factory",
          "name": "factory_",
          "type": "address"
        },
        {
          "internalType": "contract INonfungiblePositionManager",
          "name": "nftManager_",
          "type": "address"
        }
      ],
      "stateMutability": "nonpayable",
      "type": "constructor"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "address",
          "name": "owner",
          "type": "address"
        },
        {
          "indexed": true,
          "internalType": "address",
          "name": "spender",
          "type": "address"
        },
        {
          "indexed": false,
          "internalType": "uint256",
          "name": "value",
          "type": "uint256"
        }
      ],
      "name": "Approval",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": false,
          "internalType": "uint8",
          "name": "version",
          "type": "uint8"
        }
      ],
      "name": "Initialized",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "address",
          "name": "user",
          "type": "address"
        },
        {
          "indexed": false,
          "internalType": "uint256",
          "name": "burnAmount0",
          "type": "uint256"
        },
        {
          "indexed": false,
          "internalType": "uint256",
          "name": "burnAmount1",
          "type": "uint256"
        }
      ],
      "name": "LPBurned",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": false,
          "internalType": "uint24[]",
          "name": "feeTiers",
          "type": "uint24[]"
        }
      ],
      "name": "LogAddPools",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": false,
          "internalType": "address[]",
          "name": "routers",
          "type": "address[]"
        }
      ],
      "name": "LogBlacklistRouters",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "address",
          "name": "receiver",
          "type": "address"
        },
        {
          "indexed": false,
          "internalType": "uint256",
          "name": "burnAmount",
          "type": "uint256"
        },
        {
          "indexed": false,
          "internalType": "uint256",
          "name": "amount0Out",
          "type": "uint256"
        },
        {
          "indexed": false,
          "internalType": "uint256",
          "name": "amount1Out",
          "type": "uint256"
        }
      ],
      "name": "LogBurn",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": false,
          "internalType": "uint256",
          "name": "fee0",
          "type": "uint256"
        },
        {
          "indexed": false,
          "internalType": "uint256",
          "name": "fee1",
          "type": "uint256"
        }
      ],
      "name": "LogCollectedFees",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "address",
          "name": "receiver",
          "type": "address"
        },
        {
          "indexed": false,
          "internalType": "uint256",
          "name": "mintAmount",
          "type": "uint256"
        },
        {
          "indexed": false,
          "internalType": "uint256",
          "name": "amount0In",
          "type": "uint256"
        },
        {
          "indexed": false,
          "internalType": "uint256",
          "name": "amount1In",
          "type": "uint256"
        }
      ],
      "name": "LogMint",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "components": [
            {
              "components": [
                {
                  "internalType": "uint128",
                  "name": "liquidity",
                  "type": "uint128"
                },
                {
                  "components": [
                    {
                      "internalType": "int24",
                      "name": "lowerTick",
                      "type": "int24"
                    },
                    {
                      "internalType": "int24",
                      "name": "upperTick",
                      "type": "int24"
                    },
                    {
                      "internalType": "uint24",
                      "name": "feeTier",
                      "type": "uint24"
                    }
                  ],
                  "internalType": "struct Range",
                  "name": "range",
                  "type": "tuple"
                }
              ],
              "internalType": "struct PositionLiquidity[]",
              "name": "burns",
              "type": "tuple[]"
            },
            {
              "components": [
                {
                  "internalType": "uint128",
                  "name": "liquidity",
                  "type": "uint128"
                },
                {
                  "components": [
                    {
                      "internalType": "int24",
                      "name": "lowerTick",
                      "type": "int24"
                    },
                    {
                      "internalType": "int24",
                      "name": "upperTick",
                      "type": "int24"
                    },
                    {
                      "internalType": "uint24",
                      "name": "feeTier",
                      "type": "uint24"
                    }
                  ],
                  "internalType": "struct Range",
                  "name": "range",
                  "type": "tuple"
                }
              ],
              "internalType": "struct PositionLiquidity[]",
              "name": "mints",
              "type": "tuple[]"
            },
            {
              "components": [
                {
                  "internalType": "bytes",
                  "name": "payload",
                  "type": "bytes"
                },
                {
                  "internalType": "address",
                  "name": "router",
                  "type": "address"
                },
                {
                  "internalType": "uint256",
                  "name": "amountIn",
                  "type": "uint256"
                },
                {
                  "internalType": "uint256",
                  "name": "expectedMinReturn",
                  "type": "uint256"
                },
                {
                  "internalType": "bool",
                  "name": "zeroForOne",
                  "type": "bool"
                }
              ],
              "internalType": "struct SwapPayload",
              "name": "swap",
              "type": "tuple"
            },
            {
              "internalType": "uint256",
              "name": "minBurn0",
              "type": "uint256"
            },
            {
              "internalType": "uint256",
              "name": "minBurn1",
              "type": "uint256"
            },
            {
              "internalType": "uint256",
              "name": "minDeposit0",
              "type": "uint256"
            },
            {
              "internalType": "uint256",
              "name": "minDeposit1",
              "type": "uint256"
            }
          ],
          "indexed": false,
          "internalType": "struct Rebalance",
          "name": "rebalanceParams",
          "type": "tuple"
        },
        {
          "indexed": false,
          "internalType": "uint256",
          "name": "swapDelta0",
          "type": "uint256"
        },
        {
          "indexed": false,
          "internalType": "uint256",
          "name": "swapDelta1",
          "type": "uint256"
        }
      ],
      "name": "LogRebalance",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": false,
          "internalType": "address[]",
          "name": "pools",
          "type": "address[]"
        }
      ],
      "name": "LogRemovePools",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": false,
          "internalType": "address",
          "name": "minter",
          "type": "address"
        }
      ],
      "name": "LogRestrictedMint",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": false,
          "internalType": "uint256",
          "name": "init0",
          "type": "uint256"
        },
        {
          "indexed": false,
          "internalType": "uint256",
          "name": "init1",
          "type": "uint256"
        }
      ],
      "name": "LogSetInits",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": false,
          "internalType": "address",
          "name": "newManager",
          "type": "address"
        }
      ],
      "name": "LogSetManager",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": false,
          "internalType": "uint16",
          "name": "managerFeeBPS",
          "type": "uint16"
        }
      ],
      "name": "LogSetManagerFeeBPS",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": false,
          "internalType": "address[]",
          "name": "routers",
          "type": "address[]"
        }
      ],
      "name": "LogWhitelistRouters",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": false,
          "internalType": "uint256",
          "name": "amount0",
          "type": "uint256"
        },
        {
          "indexed": false,
          "internalType": "uint256",
          "name": "amount1",
          "type": "uint256"
        }
      ],
      "name": "LogWithdrawManagerBalance",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "address",
          "name": "previousOwner",
          "type": "address"
        },
        {
          "indexed": true,
          "internalType": "address",
          "name": "newOwner",
          "type": "address"
        }
      ],
      "name": "OwnershipTransferred",
      "type": "event"
    },
    {
      "anonymous": false,
      "inputs": [
        {
          "indexed": true,
          "internalType": "address",
          "name": "from",
          "type": "address"
        },
        {
          "indexed": true,
          "internalType": "address",
          "name": "to",
          "type": "address"
        },
        {
          "indexed": false,
          "internalType": "uint256",
          "name": "value",
          "type": "uint256"
        }
      ],
      "name": "Transfer",
      "type": "event"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "owner",
          "type": "address"
        },
        {
          "internalType": "address",
          "name": "spender",
          "type": "address"
        }
      ],
      "name": "allowance",
      "outputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "spender",
          "type": "address"
        },
        {
          "internalType": "uint256",
          "name": "amount",
          "type": "uint256"
        }
      ],
      "name": "approve",
      "outputs": [
        {
          "internalType": "bool",
          "name": "",
          "type": "bool"
        }
      ],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "account",
          "type": "address"
        }
      ],
      "name": "balanceOf",
      "outputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "uint256",
          "name": "burnAmount_",
          "type": "uint256"
        },
        {
          "internalType": "uint256",
          "name": "amount0Min_",
          "type": "uint256"
        },
        {
          "internalType": "uint256",
          "name": "amount1Min_",
          "type": "uint256"
        },
        {
          "internalType": "address",
          "name": "receiver_",
          "type": "address"
        },
        {
          "internalType": "address",
          "name": "tcvRouter_",
          "type": "address"
        }
      ],
      "name": "burn",
      "outputs": [
        {
          "internalType": "uint256",
          "name": "amount0",
          "type": "uint256"
        },
        {
          "internalType": "uint256",
          "name": "amount1",
          "type": "uint256"
        }
      ],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "account",
          "type": "address"
        }
      ],
      "name": "current",
      "outputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "account",
          "type": "address"
        }
      ],
      "name": "currentNow",
      "outputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "decimals",
      "outputs": [
        {
          "internalType": "uint8",
          "name": "",
          "type": "uint8"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "spender",
          "type": "address"
        },
        {
          "internalType": "uint256",
          "name": "subtractedValue",
          "type": "uint256"
        }
      ],
      "name": "decreaseAllowance",
      "outputs": [
        {
          "internalType": "bool",
          "name": "",
          "type": "bool"
        }
      ],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "factory",
      "outputs": [
        {
          "internalType": "contract IUniswapV3Factory",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "name": "fee0RatioLiq0",
      "outputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "name": "fee1RatioLiq0",
      "outputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "getPools",
      "outputs": [
        {
          "internalType": "address[]",
          "name": "",
          "type": "address[]"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "getRanges",
      "outputs": [
        {
          "components": [
            {
              "internalType": "int24",
              "name": "lowerTick",
              "type": "int24"
            },
            {
              "internalType": "int24",
              "name": "upperTick",
              "type": "int24"
            },
            {
              "internalType": "uint24",
              "name": "feeTier",
              "type": "uint24"
            }
          ],
          "internalType": "struct Range[]",
          "name": "",
          "type": "tuple[]"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "spender",
          "type": "address"
        },
        {
          "internalType": "uint256",
          "name": "addedValue",
          "type": "uint256"
        }
      ],
      "name": "increaseAllowance",
      "outputs": [
        {
          "internalType": "bool",
          "name": "",
          "type": "bool"
        }
      ],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "account",
          "type": "address"
        }
      ],
      "name": "indexRebalance",
      "outputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "init0",
      "outputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "init1",
      "outputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "string",
          "name": "name_",
          "type": "string"
        },
        {
          "internalType": "string",
          "name": "symbol_",
          "type": "string"
        },
        {
          "components": [
            {
              "internalType": "uint24[]",
              "name": "feeTiers",
              "type": "uint24[]"
            },
            {
              "internalType": "address",
              "name": "token0",
              "type": "address"
            },
            {
              "internalType": "address",
              "name": "token1",
              "type": "address"
            },
            {
              "internalType": "address",
              "name": "owner",
              "type": "address"
            },
            {
              "internalType": "uint256",
              "name": "init0",
              "type": "uint256"
            },
            {
              "internalType": "uint256",
              "name": "init1",
              "type": "uint256"
            },
            {
              "internalType": "address",
              "name": "manager",
              "type": "address"
            },
            {
              "internalType": "address[]",
              "name": "routers",
              "type": "address[]"
            }
          ],
          "internalType": "struct InitializePayload",
          "name": "params_",
          "type": "tuple"
        }
      ],
      "name": "initialize",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "liquidityBase",
      "outputs": [
        {
          "internalType": "uint128",
          "name": "",
          "type": "uint128"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "manager",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "managerBalance0",
      "outputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "managerBalance1",
      "outputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "managerFeeBPS",
      "outputs": [
        {
          "internalType": "uint16",
          "name": "",
          "type": "uint16"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "uint256",
          "name": "mintAmount_",
          "type": "uint256"
        },
        {
          "internalType": "address",
          "name": "receiver_",
          "type": "address"
        }
      ],
      "name": "mint",
      "outputs": [
        {
          "internalType": "uint256",
          "name": "amount0",
          "type": "uint256"
        },
        {
          "internalType": "uint256",
          "name": "amount1",
          "type": "uint256"
        }
      ],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "name",
      "outputs": [
        {
          "internalType": "string",
          "name": "",
          "type": "string"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "nftManager",
      "outputs": [
        {
          "internalType": "contract INonfungiblePositionManager",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "numOfRebalance",
      "outputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "owner",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "name": "rangeToTokenIds",
      "outputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "name": "ratioLPRebalance",
      "outputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [
        {
          "components": [
            {
              "components": [
                {
                  "internalType": "uint128",
                  "name": "liquidity",
                  "type": "uint128"
                },
                {
                  "components": [
                    {
                      "internalType": "int24",
                      "name": "lowerTick",
                      "type": "int24"
                    },
                    {
                      "internalType": "int24",
                      "name": "upperTick",
                      "type": "int24"
                    },
                    {
                      "internalType": "uint24",
                      "name": "feeTier",
                      "type": "uint24"
                    }
                  ],
                  "internalType": "struct Range",
                  "name": "range",
                  "type": "tuple"
                }
              ],
              "internalType": "struct PositionLiquidity[]",
              "name": "burns",
              "type": "tuple[]"
            },
            {
              "components": [
                {
                  "internalType": "uint128",
                  "name": "liquidity",
                  "type": "uint128"
                },
                {
                  "components": [
                    {
                      "internalType": "int24",
                      "name": "lowerTick",
                      "type": "int24"
                    },
                    {
                      "internalType": "int24",
                      "name": "upperTick",
                      "type": "int24"
                    },
                    {
                      "internalType": "uint24",
                      "name": "feeTier",
                      "type": "uint24"
                    }
                  ],
                  "internalType": "struct Range",
                  "name": "range",
                  "type": "tuple"
                }
              ],
              "internalType": "struct PositionLiquidity[]",
              "name": "mints",
              "type": "tuple[]"
            },
            {
              "components": [
                {
                  "internalType": "bytes",
                  "name": "payload",
                  "type": "bytes"
                },
                {
                  "internalType": "address",
                  "name": "router",
                  "type": "address"
                },
                {
                  "internalType": "uint256",
                  "name": "amountIn",
                  "type": "uint256"
                },
                {
                  "internalType": "uint256",
                  "name": "expectedMinReturn",
                  "type": "uint256"
                },
                {
                  "internalType": "bool",
                  "name": "zeroForOne",
                  "type": "bool"
                }
              ],
              "internalType": "struct SwapPayload",
              "name": "swap",
              "type": "tuple"
            },
            {
              "internalType": "uint256",
              "name": "minBurn0",
              "type": "uint256"
            },
            {
              "internalType": "uint256",
              "name": "minBurn1",
              "type": "uint256"
            },
            {
              "internalType": "uint256",
              "name": "minDeposit0",
              "type": "uint256"
            },
            {
              "internalType": "uint256",
              "name": "minDeposit1",
              "type": "uint256"
            }
          ],
          "internalType": "struct Rebalance",
          "name": "rebalanceParams_",
          "type": "tuple"
        }
      ],
      "name": "rebalance",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "account",
          "type": "address"
        }
      ],
      "name": "remainFee0User",
      "outputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "account",
          "type": "address"
        }
      ],
      "name": "remainFee1User",
      "outputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "renounceOwnership",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "restrictedMint",
      "outputs": [
        {
          "internalType": "address",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "account",
          "type": "address"
        },
        {
          "internalType": "uint256",
          "name": "amount",
          "type": "uint256"
        }
      ],
      "name": "setCurrent",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "account",
          "type": "address"
        },
        {
          "internalType": "uint256",
          "name": "index",
          "type": "uint256"
        }
      ],
      "name": "setIndexRebelance",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "setLiquidity",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "manager_",
          "type": "address"
        }
      ],
      "name": "setManager",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "uint16",
          "name": "managerFeeBPS_",
          "type": "uint16"
        }
      ],
      "name": "setManagerFeeBPS",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "uint256",
          "name": "index",
          "type": "uint256"
        }
      ],
      "name": "setNumOfRebalance",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "symbol",
      "outputs": [
        {
          "internalType": "string",
          "name": "",
          "type": "string"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "token0",
      "outputs": [
        {
          "internalType": "contract IERC20",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "token1",
      "outputs": [
        {
          "internalType": "contract IERC20",
          "name": "",
          "type": "address"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "totalLiquidity",
      "outputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "totalLiquidity0",
      "outputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "totalLiquidityNFT",
      "outputs": [
        {
          "internalType": "uint128",
          "name": "",
          "type": "uint128"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "totalSupply",
      "outputs": [
        {
          "internalType": "uint256",
          "name": "",
          "type": "uint256"
        }
      ],
      "stateMutability": "view",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "to",
          "type": "address"
        },
        {
          "internalType": "uint256",
          "name": "amount",
          "type": "uint256"
        }
      ],
      "name": "transfer",
      "outputs": [
        {
          "internalType": "bool",
          "name": "",
          "type": "bool"
        }
      ],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "from",
          "type": "address"
        },
        {
          "internalType": "address",
          "name": "to",
          "type": "address"
        },
        {
          "internalType": "uint256",
          "name": "amount",
          "type": "uint256"
        }
      ],
      "name": "transferFrom",
      "outputs": [
        {
          "internalType": "bool",
          "name": "",
          "type": "bool"
        }
      ],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [
        {
          "internalType": "address",
          "name": "newOwner",
          "type": "address"
        }
      ],
      "name": "transferOwnership",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    },
    {
      "inputs": [],
      "name": "withdrawManagerBalance",
      "outputs": [],
      "stateMutability": "nonpayable",
      "type": "function"
    }
  ]
''')


import json

UNISWAP_FACTORY_ABI =[{
                                      "inputs": [
                                          {"internalType": "address", "name": "_feeToSetter", "type": "address"}],
                                      "payable": False, "stateMutability": "nonpayable", "type": "constructor"}, {
                                      "anonymous": False, "inputs": [
        {"indexed": True, "internalType": "address", "name": "token0", "type": "address"},
        {"indexed": True, "internalType": "address", "name": "token1", "type": "address"},
        {"indexed": False, "internalType": "address", "name": "pair", "type": "address"},
        {"indexed": False, "internalType": "uint256", "name": "", "type": "uint256"}], "name": "PairCreated",
                                      "type": "event"}, {
                                      "constant": True,
                                      "inputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                      "name": "allPairs",
                                      "outputs": [{"internalType": "address", "name": "", "type": "address"}],
                                      "payable": False, "stateMutability": "view", "type": "function"}, {
                                      "constant": True, "inputs": [], "name": "allPairsLength",
                                      "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
                                      "payable": False, "stateMutability": "view", "type": "function"}, {
                                      "constant": False,
                                      "inputs": [{"internalType": "address", "name": "tokenA", "type": "address"},
                                                 {"internalType": "address", "name": "tokenB", "type": "address"}],
                                      "name": "createPair",
                                      "outputs": [{"internalType": "address", "name": "pair", "type": "address"}],
                                      "payable": False, "stateMutability": "nonpayable", "type": "function"}, {
                                      "constant": True, "inputs": [], "name": "feeTo",
                                      "outputs": [{"internalType": "address", "name": "", "type": "address"}],
                                      "payable": False, "stateMutability": "view", "type": "function"}, {
                                      "constant": True, "inputs": [], "name": "feeToSetter",
                                      "outputs": [{"internalType": "address", "name": "", "type": "address"}],
                                      "payable": False, "stateMutability": "view", "type": "function"}, {
                                      "constant": True,
                                      "inputs": [{"internalType": "address", "name": "", "type": "address"},
                                                 {"internalType": "address", "name": "", "type": "address"}],
                                      "name": "getPair",
                                      "outputs": [{"internalType": "address", "name": "", "type": "address"}],
                                      "payable": False, "stateMutability": "view", "type": "function"}, {
                                      "constant": False,
                                      "inputs": [{"internalType": "address", "name": "_feeTo", "type": "address"}],
                                      "name": "setFeeTo", "outputs": [], "payable": False,
                                      "stateMutability": "nonpayable", "type": "function"}, {
                                      "constant": False, "inputs": [
        {"internalType": "address", "name": "_feeToSetter", "type": "address"}], "name": "setFeeToSetter",
                                      "outputs": [], "payable": False, "stateMutability": "nonpayable",
                                      "type": "function"}]




import json

COMPTROLLERBEACON = json.loads("""
[
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "implementation_",
        "type": "address"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "constructor"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "previousOwner",
        "type": "address"
      },
      {
        "indexed": true,
        "internalType": "address",
        "name": "newOwner",
        "type": "address"
      }
    ],
    "name": "OwnershipTransferred",
    "type": "event"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": true,
        "internalType": "address",
        "name": "implementation",
        "type": "address"
      }
    ],
    "name": "Upgraded",
    "type": "event"
  },
  {
    "inputs": [],
    "name": "implementation",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "owner",
    "outputs": [
      {
        "internalType": "address",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "renounceOwnership",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "newOwner",
        "type": "address"
      }
    ],
    "name": "transferOwnership",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "newImplementation",
        "type": "address"
      }
    ],
    "name": "upgradeTo",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  }
]
""")
        


import json
from urllib.parse import quote
import requests
import base64

from cosmpy.cosmwasm.rest_client import RestClient
from cosmpy.cosmwasm.rest_client import CosmWasmRestClient
from cosmpy.protos.cosmwasm.wasm.v1.query_pb2 import QuerySmartContractStateRequest

from defi_services.constants.cosmos_decimals_constant import Denoms, MulticallContract


class CosmosTokenServices:
    def __init__(self, lcd: str, rest_uri: str, chain_id: str):
        self.chain_id = chain_id
        self.lcd = lcd

        self.rest_uri = rest_uri
        self.client = CosmWasmRestClient(RestClient(rest_address=rest_uri))

        self.decimals = Denoms.cosmos
        self.decimals.update(Denoms.orai)

    # queries the balance of all coins for a single account.
    def query_coin_balances(self, address: str):
        responses = []
        endpoint = '/cosmos/bank/v1beta1/balances/'

        response = requests.get(self.lcd + endpoint + address, timeout=60).content
        results = json.loads(response)

        responses += results['balances']
        pagination = results['pagination']['next_key']

        while pagination is not None:
            response = requests.get(self.lcd + endpoint + '?pagination.key=' + quote(str(pagination)), timeout=60)
            results = json.loads(response.content)

            responses += results['balances']
            pagination = results['pagination']['next_key']

        return responses

    # queries the balance of a given denom for a single account.
    def query_balances_by_denom(self, address, denom):
        endpoint = '/cosmos/bank/v1beta1/balances/'

        response = requests.get(self.lcd + endpoint + address + '/by_denom?denom=' + denom, timeout=60)
        return json.loads(response.content)

    def query_token_decimal(self):
        endpoint = '/cosmos/bank/v1beta1/denoms_metadata'

        response = requests.get(self.lcd + endpoint, timeout=60)
        return json.loads(response.content)

    def query_balances(self, address: str, tokens: list):
        balance_query, cw20_tokens = self.get_balance_function_info(address, tokens)
        queries = []
        for token in cw20_tokens:
            queries.append({
                "address": token,
                "data": self.encode_data({"token_info": {}})
            })
        queries.append({
            "address": MulticallContract.mapping.get(self.chain_id).get("multicall_balance"),
            "data": self.encode_data(balance_query)
        })
        query = {
            "aggregate": {
                "queries": queries
            }
        }
        query = json.dumps(query).encode('utf-8')
        request_ = QuerySmartContractStateRequest(
            address=MulticallContract.mapping.get(self.chain_id).get("multicall"), query_data=query)
        response_ = self.client.SmartContractState(request_)
        decoded_data = self.decode_response_data(response_, cw20_tokens, tokens)
        return decoded_data

    def decode_response_data(self, response_data, cw20_tokens, tokens):
        response_data = json.loads(response_data.data)
        return_data = response_data.get("return_data", [])
        if not len(return_data):
            return None
        decimals = {}
        for idx in range(0, len(cw20_tokens)):
            encoded_data = return_data[idx]
            decoded_data = self.decode_data(encoded_data['data'])
            decimals[cw20_tokens[idx]] = decoded_data.get("decimals")

        balance_data = self.decode_data(return_data[-1]['data'])
        balances = {}
        for idx in range(0, len(tokens)):
            balances[tokens[idx]] = int(balance_data[idx])

        for token in tokens:
            if decimals.get(token) is not None:
                balances[token] /= 10**decimals[token]
            else:
                balances[token] /= 10**self.decimals.get(token, {}).get("decimal", 0)

        return balances

    @staticmethod
    def encode_data(params: dict):
        data = json.dumps(params).encode('utf-8')
        encode_data = base64.b64encode(data).decode()
        return encode_data

    @staticmethod
    def decode_data(encode_data: str):
        decoded_data = base64.b64decode(encode_data)
        decoded_data = json.loads(decoded_data)
        return decoded_data

    @staticmethod
    def get_balance_function_info(address: str, tokens: list):
        data = {
            "balance": {
                "address": address,
                "assets": []
            }
        }
        cw20_tokens = []
        for token in tokens:
            head = token[:4]
            if len(token) < 10 or head == 'ibc/':
                data["balance"]["assets"].append({
                    "native_token": {
                        "denom": token
                    }
                })
            else:
                data["balance"]["assets"].append(
                    {
                        "token": {
                            "contract_addr": token
                        }
                    }
                )
                cw20_tokens.append(token)
        return data, cw20_tokens




import json

JUMPRATEMODELV2 = json.loads("""
[
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "baseRatePerYear",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "multiplierPerYear",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "jumpMultiplierPerYear",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "kink_",
        "type": "uint256"
      },
      {
        "internalType": "contract IAccessControlManagerV8",
        "name": "accessControlManager_",
        "type": "address"
      }
    ],
    "stateMutability": "nonpayable",
    "type": "constructor"
  },
  {
    "inputs": [
      {
        "internalType": "address",
        "name": "sender",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "calledContract",
        "type": "address"
      },
      {
        "internalType": "string",
        "name": "methodSignature",
        "type": "string"
      }
    ],
    "name": "Unauthorized",
    "type": "error"
  },
  {
    "anonymous": false,
    "inputs": [
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "baseRatePerBlock",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "multiplierPerBlock",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "jumpMultiplierPerBlock",
        "type": "uint256"
      },
      {
        "indexed": false,
        "internalType": "uint256",
        "name": "kink",
        "type": "uint256"
      }
    ],
    "name": "NewInterestParams",
    "type": "event"
  },
  {
    "inputs": [],
    "name": "accessControlManager",
    "outputs": [
      {
        "internalType": "contract IAccessControlManagerV8",
        "name": "",
        "type": "address"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "baseRatePerBlock",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "cash",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "borrows",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "reserves",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "badDebt",
        "type": "uint256"
      }
    ],
    "name": "getBorrowRate",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "cash",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "borrows",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "reserves",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "reserveFactorMantissa",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "badDebt",
        "type": "uint256"
      }
    ],
    "name": "getSupplyRate",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "isInterestRateModel",
    "outputs": [
      {
        "internalType": "bool",
        "name": "",
        "type": "bool"
      }
    ],
    "stateMutability": "pure",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "jumpMultiplierPerBlock",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "kink",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "multiplierPerBlock",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "baseRatePerYear",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "multiplierPerYear",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "jumpMultiplierPerYear",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "kink_",
        "type": "uint256"
      }
    ],
    "name": "updateJumpRateModel",
    "outputs": [],
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "cash",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "borrows",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "reserves",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "badDebt",
        "type": "uint256"
      }
    ],
    "name": "utilizationRate",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "pure",
    "type": "function"
  }
]
""")
        


import json

STRIKE_LENS_ABI = json.loads('''
[
  {
    "constant": false,
    "inputs": [
      {
        "internalType": "contract ComptrollerLensInterface",
        "name": "comptroller",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "getAccountLimits",
    "outputs": [
      {
        "components": [
          {
            "internalType": "contract SToken[]",
            "name": "markets",
            "type": "address[]"
          },
          {
            "internalType": "uint256",
            "name": "liquidity",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "shortfall",
            "type": "uint256"
          }
        ],
        "internalType": "struct StrikeLens.AccountLimits",
        "name": "",
        "type": "tuple"
      }
    ],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "constant": true,
    "inputs": [
      {
        "internalType": "contract GovernorAlpha",
        "name": "governor",
        "type": "address"
      },
      {
        "internalType": "uint256[]",
        "name": "proposalIds",
        "type": "uint256[]"
      }
    ],
    "name": "getGovProposals",
    "outputs": [
      {
        "components": [
          {
            "internalType": "uint256",
            "name": "proposalId",
            "type": "uint256"
          },
          {
            "internalType": "address",
            "name": "proposer",
            "type": "address"
          },
          {
            "internalType": "uint256",
            "name": "eta",
            "type": "uint256"
          },
          {
            "internalType": "address[]",
            "name": "targets",
            "type": "address[]"
          },
          {
            "internalType": "uint256[]",
            "name": "values",
            "type": "uint256[]"
          },
          {
            "internalType": "string[]",
            "name": "signatures",
            "type": "string[]"
          },
          {
            "internalType": "bytes[]",
            "name": "calldatas",
            "type": "bytes[]"
          },
          {
            "internalType": "uint256",
            "name": "startBlock",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "endBlock",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "forVotes",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "againstVotes",
            "type": "uint256"
          },
          {
            "internalType": "bool",
            "name": "canceled",
            "type": "bool"
          },
          {
            "internalType": "bool",
            "name": "executed",
            "type": "bool"
          }
        ],
        "internalType": "struct StrikeLens.GovProposal[]",
        "name": "",
        "type": "tuple[]"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function"
  },
  {
    "constant": true,
    "inputs": [
      {
        "internalType": "contract GovernorAlpha",
        "name": "governor",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "voter",
        "type": "address"
      },
      {
        "internalType": "uint256[]",
        "name": "proposalIds",
        "type": "uint256[]"
      }
    ],
    "name": "getGovReceipts",
    "outputs": [
      {
        "components": [
          {
            "internalType": "uint256",
            "name": "proposalId",
            "type": "uint256"
          },
          {
            "internalType": "bool",
            "name": "hasVoted",
            "type": "bool"
          },
          {
            "internalType": "bool",
            "name": "support",
            "type": "bool"
          },
          {
            "internalType": "uint96",
            "name": "votes",
            "type": "uint96"
          }
        ],
        "internalType": "struct StrikeLens.GovReceipt[]",
        "name": "",
        "type": "tuple[]"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function"
  },
  {
    "constant": true,
    "inputs": [
      {
        "internalType": "contract STRK",
        "name": "strk",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "getStrikeBalanceMetadata",
    "outputs": [
      {
        "components": [
          {
            "internalType": "uint256",
            "name": "balance",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "votes",
            "type": "uint256"
          },
          {
            "internalType": "address",
            "name": "delegate",
            "type": "address"
          }
        ],
        "internalType": "struct StrikeLens.StrikeBalanceMetadata",
        "name": "",
        "type": "tuple"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function"
  },
  {
    "constant": false,
    "inputs": [
      {
        "internalType": "contract STRK",
        "name": "strk",
        "type": "address"
      },
      {
        "internalType": "contract ComptrollerLensInterface",
        "name": "comptroller",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "getStrikeBalanceMetadataExt",
    "outputs": [
      {
        "components": [
          {
            "internalType": "uint256",
            "name": "balance",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "votes",
            "type": "uint256"
          },
          {
            "internalType": "address",
            "name": "delegate",
            "type": "address"
          },
          {
            "internalType": "uint256",
            "name": "allocated",
            "type": "uint256"
          }
        ],
        "internalType": "struct StrikeLens.StrikeBalanceMetadataExt",
        "name": "",
        "type": "tuple"
      }
    ],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "constant": true,
    "inputs": [
      {
        "internalType": "contract STRK",
        "name": "strk",
        "type": "address"
      },
      {
        "internalType": "address",
        "name": "account",
        "type": "address"
      },
      {
        "internalType": "uint32[]",
        "name": "blockNumbers",
        "type": "uint32[]"
      }
    ],
    "name": "getStrikeVotes",
    "outputs": [
      {
        "components": [
          {
            "internalType": "uint256",
            "name": "blockNumber",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "votes",
            "type": "uint256"
          }
        ],
        "internalType": "struct StrikeLens.StrikeVotes[]",
        "name": "",
        "type": "tuple[]"
      }
    ],
    "payable": false,
    "stateMutability": "view",
    "type": "function"
  },
  {
    "constant": false,
    "inputs": [
      {
        "internalType": "contract SToken",
        "name": "sToken",
        "type": "address"
      },
      {
        "internalType": "address payable",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "sTokenBalances",
    "outputs": [
      {
        "components": [
          {
            "internalType": "address",
            "name": "sToken",
            "type": "address"
          },
          {
            "internalType": "uint256",
            "name": "balanceOf",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "borrowBalanceCurrent",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "balanceOfUnderlying",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "tokenBalance",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "tokenAllowance",
            "type": "uint256"
          }
        ],
        "internalType": "struct StrikeLens.STokenBalances",
        "name": "",
        "type": "tuple"
      }
    ],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "constant": false,
    "inputs": [
      {
        "internalType": "contract SToken[]",
        "name": "sTokens",
        "type": "address[]"
      },
      {
        "internalType": "address payable",
        "name": "account",
        "type": "address"
      }
    ],
    "name": "sTokenBalancesAll",
    "outputs": [
      {
        "components": [
          {
            "internalType": "address",
            "name": "sToken",
            "type": "address"
          },
          {
            "internalType": "uint256",
            "name": "balanceOf",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "borrowBalanceCurrent",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "balanceOfUnderlying",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "tokenBalance",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "tokenAllowance",
            "type": "uint256"
          }
        ],
        "internalType": "struct StrikeLens.STokenBalances[]",
        "name": "",
        "type": "tuple[]"
      }
    ],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "constant": false,
    "inputs": [
      {
        "internalType": "contract SToken",
        "name": "sToken",
        "type": "address"
      }
    ],
    "name": "sTokenMetadata",
    "outputs": [
      {
        "components": [
          {
            "internalType": "address",
            "name": "sToken",
            "type": "address"
          },
          {
            "internalType": "uint256",
            "name": "exchangeRateCurrent",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "supplyRatePerBlock",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "borrowRatePerBlock",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "reserveFactorMantissa",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "totalBorrows",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "totalReserves",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "totalSupply",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "totalCash",
            "type": "uint256"
          },
          {
            "internalType": "bool",
            "name": "isListed",
            "type": "bool"
          },
          {
            "internalType": "uint256",
            "name": "collateralFactorMantissa",
            "type": "uint256"
          },
          {
            "internalType": "address",
            "name": "underlyingAssetAddress",
            "type": "address"
          },
          {
            "internalType": "uint256",
            "name": "sTokenDecimals",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "underlyingDecimals",
            "type": "uint256"
          }
        ],
        "internalType": "struct StrikeLens.STokenMetadata",
        "name": "",
        "type": "tuple"
      }
    ],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "constant": false,
    "inputs": [
      {
        "internalType": "contract SToken[]",
        "name": "sTokens",
        "type": "address[]"
      }
    ],
    "name": "sTokenMetadataAll",
    "outputs": [
      {
        "components": [
          {
            "internalType": "address",
            "name": "sToken",
            "type": "address"
          },
          {
            "internalType": "uint256",
            "name": "exchangeRateCurrent",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "supplyRatePerBlock",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "borrowRatePerBlock",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "reserveFactorMantissa",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "totalBorrows",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "totalReserves",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "totalSupply",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "totalCash",
            "type": "uint256"
          },
          {
            "internalType": "bool",
            "name": "isListed",
            "type": "bool"
          },
          {
            "internalType": "uint256",
            "name": "collateralFactorMantissa",
            "type": "uint256"
          },
          {
            "internalType": "address",
            "name": "underlyingAssetAddress",
            "type": "address"
          },
          {
            "internalType": "uint256",
            "name": "sTokenDecimals",
            "type": "uint256"
          },
          {
            "internalType": "uint256",
            "name": "underlyingDecimals",
            "type": "uint256"
          }
        ],
        "internalType": "struct StrikeLens.STokenMetadata[]",
        "name": "",
        "type": "tuple[]"
      }
    ],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "constant": false,
    "inputs": [
      {
        "internalType": "contract SToken",
        "name": "sToken",
        "type": "address"
      }
    ],
    "name": "sTokenUnderlyingPrice",
    "outputs": [
      {
        "components": [
          {
            "internalType": "address",
            "name": "sToken",
            "type": "address"
          },
          {
            "internalType": "uint256",
            "name": "underlyingPrice",
            "type": "uint256"
          }
        ],
        "internalType": "struct StrikeLens.STokenUnderlyingPrice",
        "name": "",
        "type": "tuple"
      }
    ],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function"
  },
  {
    "constant": false,
    "inputs": [
      {
        "internalType": "contract SToken[]",
        "name": "sTokens",
        "type": "address[]"
      }
    ],
    "name": "sTokenUnderlyingPriceAll",
    "outputs": [
      {
        "components": [
          {
            "internalType": "address",
            "name": "sToken",
            "type": "address"
          },
          {
            "internalType": "uint256",
            "name": "underlyingPrice",
            "type": "uint256"
          }
        ],
        "internalType": "struct StrikeLens.STokenUnderlyingPrice[]",
        "name": "",
        "type": "tuple[]"
      }
    ],
    "payable": false,
    "stateMutability": "nonpayable",
    "type": "function"
  }
]
''')



import json

GHO_DISCOUNT_RATE_STRATEGY = json.loads("""
[
  {
    "inputs": [],
    "name": "DISCOUNT_RATE",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "GHO_DISCOUNTED_PER_DISCOUNT_TOKEN",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "MIN_DEBT_TOKEN_BALANCE",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [],
    "name": "MIN_DISCOUNT_TOKEN_BALANCE",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "view",
    "type": "function"
  },
  {
    "inputs": [
      {
        "internalType": "uint256",
        "name": "debtBalance",
        "type": "uint256"
      },
      {
        "internalType": "uint256",
        "name": "discountTokenBalance",
        "type": "uint256"
      }
    ],
    "name": "calculateDiscountRate",
    "outputs": [
      {
        "internalType": "uint256",
        "name": "",
        "type": "uint256"
      }
    ],
    "stateMutability": "pure",
    "type": "function"
  }
]
""")
        


from defi_services.constants.chain_constant import Chain
from defi_services.constants.entities.dex_constant import Dex
from defi_services.services.dex.dex_info.pancakeswap_info import *
from defi_services.services.dex.dex_info.quickswap_info import QUICKSWAP_POLYGON_V2_INFO
from defi_services.services.dex.dex_info.spookyswap_info import SPOOKY_FTM_V2_INFO
from defi_services.services.dex.dex_info.sushiswap_info import *
from defi_services.services.dex.dex_info.uniswap_info import *


class DexInfo:
    ethereum = {
        Dex.uniswap_v2: UNISWAP_ETH_INFO,
        Dex.uniswap_v3: UNISWAP_V3_ETH_INFO,
        Dex.sushi: SUSHISWAP_V0_ETH_INFO,
        Dex.sushi_v2: SUSHISWAP_V2_ETH_INFO,

    }
    bsc = {
        Dex.pancake: PANCAKESWAP_V0_BSC_INFO,
        Dex.pancake_v3: PANCAKESWAP_V3_BSC_INFO,
        Dex.pancake_v2: PANCAKESWAP_V2_BSC_INFO,
        Dex.sushi_v2: SUSHISWAP_V2_BSC_INFO
    }
    avalanche = {
        Dex.sushi_v2: SUSHISWAP_V2_AVALANCHE_INFO
    }
    polygon = {
        Dex.sushi_v2: SUSHISWAP_V2_POLYGON_INFO,
        Dex.quickswap_v2: QUICKSWAP_POLYGON_V2_INFO
    }
    fantom = {
        Dex.sushi_v2: SUSHISWAP_V2_FANTOM_INFO,
        Dex.spooky_v2: SPOOKY_FTM_V2_INFO
    }
    # optimism = {
    #     Dex.sushi: SushiSwapV2Services
    # }
    arbitrum = {
        Dex.sushi_v2: SUSHISWAP_V2_ARBITRUM_INFO,
        Dex.uniswap_v3: UNISWAP_V3_ARBITRUM_INFO
    }
    mapping = {
        Chain.ethereum: ethereum,
        Chain.fantom: fantom,
        Chain.bsc: bsc,
        Chain.avalanche: avalanche,
        Chain.polygon: polygon,
        Chain.arbitrum: arbitrum,
        # Chain.optimism: optimism,
    }



import json

LIDO_CURVE_STETH_ETH_GAUGE = json.loads('''
[
  {
    "name": "Deposit",
    "inputs": [
      {
        "name": "provider",
        "type": "address",
        "indexed": true
      },
      {
        "name": "value",
        "type": "uint256",
        "indexed": false
      }
    ],
    "anonymous": false,
    "type": "event"
  },
  {
    "name": "Withdraw",
    "inputs": [
      {
        "name": "provider",
        "type": "address",
        "indexed": true
      },
      {
        "name": "value",
        "type": "uint256",
        "indexed": false
      }
    ],
    "anonymous": false,
    "type": "event"
  },
  {
    "name": "UpdateLiquidityLimit",
    "inputs": [
      {
        "name": "user",
        "type": "address",
        "indexed": false
      },
      {
        "name": "original_balance",
        "type": "uint256",
        "indexed": false
      },
      {
        "name": "original_supply",
        "type": "uint256",
        "indexed": false
      },
      {
        "name": "working_balance",
        "type": "uint256",
        "indexed": false
      },
      {
        "name": "working_supply",
        "type": "uint256",
        "indexed": false
      }
    ],
    "anonymous": false,
    "type": "event"
  },
  {
    "name": "CommitOwnership",
    "inputs": [
      {
        "name": "admin",
        "type": "address",
        "indexed": false
      }
    ],
    "anonymous": false,
    "type": "event"
  },
  {
    "name": "ApplyOwnership",
    "inputs": [
      {
        "name": "admin",
        "type": "address",
        "indexed": false
      }
    ],
    "anonymous": false,
    "type": "event"
  },
  {
    "name": "Transfer",
    "inputs": [
      {
        "name": "_from",
        "type": "address",
        "indexed": true
      },
      {
        "name": "_to",
        "type": "address",
        "indexed": true
      },
      {
        "name": "_value",
        "type": "uint256",
        "indexed": false
      }
    ],
    "anonymous": false,
    "type": "event"
  },
  {
    "name": "Approval",
    "inputs": [
      {
        "name": "_owner",
        "type": "address",
        "indexed": true
      },
      {
        "name": "_spender",
        "type": "address",
        "indexed": true
      },
      {
        "name": "_value",
        "type": "uint256",
        "indexed": false
      }
    ],
    "anonymous": false,
    "type": "event"
  },
  {
    "stateMutability": "nonpayable",
    "type": "constructor",
    "inputs": [],
    "outputs": []
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "initialize",
    "inputs": [
      {
        "name": "_lp_token",
        "type": "address"
      }
    ],
    "outputs": [],
    "gas": 374587
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "decimals",
    "inputs": [],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 318
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "integrate_checkpoint",
    "inputs": [],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 4590
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "user_checkpoint",
    "inputs": [
      {
        "name": "addr",
        "type": "address"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "bool"
      }
    ],
    "gas": 3123886
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "claimable_tokens",
    "inputs": [
      {
        "name": "addr",
        "type": "address"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 3038676
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "claimed_reward",
    "inputs": [
      {
        "name": "_addr",
        "type": "address"
      },
      {
        "name": "_token",
        "type": "address"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 3036
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "claimable_reward",
    "inputs": [
      {
        "name": "_user",
        "type": "address"
      },
      {
        "name": "_reward_token",
        "type": "address"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 20255
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "set_rewards_receiver",
    "inputs": [
      {
        "name": "_receiver",
        "type": "address"
      }
    ],
    "outputs": [],
    "gas": 35673
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "claim_rewards",
    "inputs": [],
    "outputs": []
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "claim_rewards",
    "inputs": [
      {
        "name": "_addr",
        "type": "address"
      }
    ],
    "outputs": []
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "claim_rewards",
    "inputs": [
      {
        "name": "_addr",
        "type": "address"
      },
      {
        "name": "_receiver",
        "type": "address"
      }
    ],
    "outputs": []
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "kick",
    "inputs": [
      {
        "name": "addr",
        "type": "address"
      }
    ],
    "outputs": [],
    "gas": 3137977
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "deposit",
    "inputs": [
      {
        "name": "_value",
        "type": "uint256"
      }
    ],
    "outputs": []
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "deposit",
    "inputs": [
      {
        "name": "_value",
        "type": "uint256"
      },
      {
        "name": "_addr",
        "type": "address"
      }
    ],
    "outputs": []
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "deposit",
    "inputs": [
      {
        "name": "_value",
        "type": "uint256"
      },
      {
        "name": "_addr",
        "type": "address"
      },
      {
        "name": "_claim_rewards",
        "type": "bool"
      }
    ],
    "outputs": []
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "withdraw",
    "inputs": [
      {
        "name": "_value",
        "type": "uint256"
      }
    ],
    "outputs": []
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "withdraw",
    "inputs": [
      {
        "name": "_value",
        "type": "uint256"
      },
      {
        "name": "_claim_rewards",
        "type": "bool"
      }
    ],
    "outputs": []
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "transfer",
    "inputs": [
      {
        "name": "_to",
        "type": "address"
      },
      {
        "name": "_value",
        "type": "uint256"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "bool"
      }
    ],
    "gas": 18062826
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "transferFrom",
    "inputs": [
      {
        "name": "_from",
        "type": "address"
      },
      {
        "name": "_to",
        "type": "address"
      },
      {
        "name": "_value",
        "type": "uint256"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "bool"
      }
    ],
    "gas": 18100776
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "approve",
    "inputs": [
      {
        "name": "_spender",
        "type": "address"
      },
      {
        "name": "_value",
        "type": "uint256"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "bool"
      }
    ],
    "gas": 38151
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "increaseAllowance",
    "inputs": [
      {
        "name": "_spender",
        "type": "address"
      },
      {
        "name": "_added_value",
        "type": "uint256"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "bool"
      }
    ],
    "gas": 40695
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "decreaseAllowance",
    "inputs": [
      {
        "name": "_spender",
        "type": "address"
      },
      {
        "name": "_subtracted_value",
        "type": "uint256"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "bool"
      }
    ],
    "gas": 40719
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "add_reward",
    "inputs": [
      {
        "name": "_reward_token",
        "type": "address"
      },
      {
        "name": "_distributor",
        "type": "address"
      }
    ],
    "outputs": [],
    "gas": 115414
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "set_reward_distributor",
    "inputs": [
      {
        "name": "_reward_token",
        "type": "address"
      },
      {
        "name": "_distributor",
        "type": "address"
      }
    ],
    "outputs": [],
    "gas": 43179
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "deposit_reward_token",
    "inputs": [
      {
        "name": "_reward_token",
        "type": "address"
      },
      {
        "name": "_amount",
        "type": "uint256"
      }
    ],
    "outputs": [],
    "gas": 1540067
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "set_killed",
    "inputs": [
      {
        "name": "_is_killed",
        "type": "bool"
      }
    ],
    "outputs": [],
    "gas": 40529
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "lp_token",
    "inputs": [],
    "outputs": [
      {
        "name": "",
        "type": "address"
      }
    ],
    "gas": 3018
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "future_epoch_time",
    "inputs": [],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 3048
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "balanceOf",
    "inputs": [
      {
        "name": "arg0",
        "type": "address"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 3293
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "totalSupply",
    "inputs": [],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 3108
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "allowance",
    "inputs": [
      {
        "name": "arg0",
        "type": "address"
      },
      {
        "name": "arg1",
        "type": "address"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 3568
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "name",
    "inputs": [],
    "outputs": [
      {
        "name": "",
        "type": "string"
      }
    ],
    "gas": 13398
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "symbol",
    "inputs": [],
    "outputs": [
      {
        "name": "",
        "type": "string"
      }
    ],
    "gas": 11151
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "working_balances",
    "inputs": [
      {
        "name": "arg0",
        "type": "address"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 3443
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "working_supply",
    "inputs": [],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 3258
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "period",
    "inputs": [],
    "outputs": [
      {
        "name": "",
        "type": "int128"
      }
    ],
    "gas": 3288
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "period_timestamp",
    "inputs": [
      {
        "name": "arg0",
        "type": "uint256"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 3363
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "integrate_inv_supply",
    "inputs": [
      {
        "name": "arg0",
        "type": "uint256"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 3393
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "integrate_inv_supply_of",
    "inputs": [
      {
        "name": "arg0",
        "type": "address"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 3593
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "integrate_checkpoint_of",
    "inputs": [
      {
        "name": "arg0",
        "type": "address"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 3623
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "integrate_fraction",
    "inputs": [
      {
        "name": "arg0",
        "type": "address"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 3653
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "inflation_rate",
    "inputs": [],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 3468
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "reward_count",
    "inputs": [],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 3498
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "reward_tokens",
    "inputs": [
      {
        "name": "arg0",
        "type": "uint256"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "address"
      }
    ],
    "gas": 3573
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "reward_data",
    "inputs": [
      {
        "name": "arg0",
        "type": "address"
      }
    ],
    "outputs": [
      {
        "name": "token",
        "type": "address"
      },
      {
        "name": "distributor",
        "type": "address"
      },
      {
        "name": "period_finish",
        "type": "uint256"
      },
      {
        "name": "rate",
        "type": "uint256"
      },
      {
        "name": "last_update",
        "type": "uint256"
      },
      {
        "name": "integral",
        "type": "uint256"
      }
    ],
    "gas": 15003
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "rewards_receiver",
    "inputs": [
      {
        "name": "arg0",
        "type": "address"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "address"
      }
    ],
    "gas": 3803
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "reward_integral_for",
    "inputs": [
      {
        "name": "arg0",
        "type": "address"
      },
      {
        "name": "arg1",
        "type": "address"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 4048
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "is_killed",
    "inputs": [],
    "outputs": [
      {
        "name": "",
        "type": "bool"
      }
    ],
    "gas": 3648
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "factory",
    "inputs": [],
    "outputs": [
      {
        "name": "",
        "type": "address"
      }
    ],
    "gas": 3678
  }
]
''')


import json

LIDO_CURVE_STETH_ETH_POOL_ABI = json.loads('''
[
  {
    "name": "Transfer",
    "inputs": [
      {
        "name": "sender",
        "type": "address",
        "indexed": true
      },
      {
        "name": "receiver",
        "type": "address",
        "indexed": true
      },
      {
        "name": "value",
        "type": "uint256",
        "indexed": false
      }
    ],
    "anonymous": false,
    "type": "event"
  },
  {
    "name": "Approval",
    "inputs": [
      {
        "name": "owner",
        "type": "address",
        "indexed": true
      },
      {
        "name": "spender",
        "type": "address",
        "indexed": true
      },
      {
        "name": "value",
        "type": "uint256",
        "indexed": false
      }
    ],
    "anonymous": false,
    "type": "event"
  },
  {
    "name": "TokenExchange",
    "inputs": [
      {
        "name": "buyer",
        "type": "address",
        "indexed": true
      },
      {
        "name": "sold_id",
        "type": "int128",
        "indexed": false
      },
      {
        "name": "tokens_sold",
        "type": "uint256",
        "indexed": false
      },
      {
        "name": "bought_id",
        "type": "int128",
        "indexed": false
      },
      {
        "name": "tokens_bought",
        "type": "uint256",
        "indexed": false
      }
    ],
    "anonymous": false,
    "type": "event"
  },
  {
    "name": "AddLiquidity",
    "inputs": [
      {
        "name": "provider",
        "type": "address",
        "indexed": true
      },
      {
        "name": "token_amounts",
        "type": "uint256[2]",
        "indexed": false
      },
      {
        "name": "fees",
        "type": "uint256[2]",
        "indexed": false
      },
      {
        "name": "invariant",
        "type": "uint256",
        "indexed": false
      },
      {
        "name": "token_supply",
        "type": "uint256",
        "indexed": false
      }
    ],
    "anonymous": false,
    "type": "event"
  },
  {
    "name": "RemoveLiquidity",
    "inputs": [
      {
        "name": "provider",
        "type": "address",
        "indexed": true
      },
      {
        "name": "token_amounts",
        "type": "uint256[2]",
        "indexed": false
      },
      {
        "name": "fees",
        "type": "uint256[2]",
        "indexed": false
      },
      {
        "name": "token_supply",
        "type": "uint256",
        "indexed": false
      }
    ],
    "anonymous": false,
    "type": "event"
  },
  {
    "name": "RemoveLiquidityOne",
    "inputs": [
      {
        "name": "provider",
        "type": "address",
        "indexed": true
      },
      {
        "name": "token_amount",
        "type": "uint256",
        "indexed": false
      },
      {
        "name": "coin_amount",
        "type": "uint256",
        "indexed": false
      },
      {
        "name": "token_supply",
        "type": "uint256",
        "indexed": false
      }
    ],
    "anonymous": false,
    "type": "event"
  },
  {
    "name": "RemoveLiquidityImbalance",
    "inputs": [
      {
        "name": "provider",
        "type": "address",
        "indexed": true
      },
      {
        "name": "token_amounts",
        "type": "uint256[2]",
        "indexed": false
      },
      {
        "name": "fees",
        "type": "uint256[2]",
        "indexed": false
      },
      {
        "name": "invariant",
        "type": "uint256",
        "indexed": false
      },
      {
        "name": "token_supply",
        "type": "uint256",
        "indexed": false
      }
    ],
    "anonymous": false,
    "type": "event"
  },
  {
    "name": "RampA",
    "inputs": [
      {
        "name": "old_A",
        "type": "uint256",
        "indexed": false
      },
      {
        "name": "new_A",
        "type": "uint256",
        "indexed": false
      },
      {
        "name": "initial_time",
        "type": "uint256",
        "indexed": false
      },
      {
        "name": "future_time",
        "type": "uint256",
        "indexed": false
      }
    ],
    "anonymous": false,
    "type": "event"
  },
  {
    "name": "StopRampA",
    "inputs": [
      {
        "name": "A",
        "type": "uint256",
        "indexed": false
      },
      {
        "name": "t",
        "type": "uint256",
        "indexed": false
      }
    ],
    "anonymous": false,
    "type": "event"
  },
  {
    "stateMutability": "nonpayable",
    "type": "constructor",
    "inputs": [],
    "outputs": []
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "initialize",
    "inputs": [
      {
        "name": "_name",
        "type": "string"
      },
      {
        "name": "_symbol",
        "type": "string"
      },
      {
        "name": "_coins",
        "type": "address[4]"
      },
      {
        "name": "_rate_multipliers",
        "type": "uint256[4]"
      },
      {
        "name": "_A",
        "type": "uint256"
      },
      {
        "name": "_fee",
        "type": "uint256"
      }
    ],
    "outputs": [],
    "gas": 471502
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "decimals",
    "inputs": [],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 318
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "transfer",
    "inputs": [
      {
        "name": "_to",
        "type": "address"
      },
      {
        "name": "_value",
        "type": "uint256"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "bool"
      }
    ],
    "gas": 77977
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "transferFrom",
    "inputs": [
      {
        "name": "_from",
        "type": "address"
      },
      {
        "name": "_to",
        "type": "address"
      },
      {
        "name": "_value",
        "type": "uint256"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "bool"
      }
    ],
    "gas": 115912
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "approve",
    "inputs": [
      {
        "name": "_spender",
        "type": "address"
      },
      {
        "name": "_value",
        "type": "uint256"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "bool"
      }
    ],
    "gas": 37851
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "balances",
    "inputs": [
      {
        "name": "i",
        "type": "uint256"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 15163
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "get_balances",
    "inputs": [],
    "outputs": [
      {
        "name": "",
        "type": "uint256[2]"
      }
    ],
    "gas": 15139
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "admin_fee",
    "inputs": [],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 498
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "A",
    "inputs": [],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 10824
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "A_precise",
    "inputs": [],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 10786
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "get_virtual_price",
    "inputs": [],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 859368
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "calc_token_amount",
    "inputs": [
      {
        "name": "_amounts",
        "type": "uint256[2]"
      },
      {
        "name": "_is_deposit",
        "type": "bool"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 3347181
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "add_liquidity",
    "inputs": [
      {
        "name": "_amounts",
        "type": "uint256[2]"
      },
      {
        "name": "_min_mint_amount",
        "type": "uint256"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ]
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "add_liquidity",
    "inputs": [
      {
        "name": "_amounts",
        "type": "uint256[2]"
      },
      {
        "name": "_min_mint_amount",
        "type": "uint256"
      },
      {
        "name": "_receiver",
        "type": "address"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ]
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "get_dy",
    "inputs": [
      {
        "name": "i",
        "type": "int128"
      },
      {
        "name": "j",
        "type": "int128"
      },
      {
        "name": "dx",
        "type": "uint256"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 2128404
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "exchange",
    "inputs": [
      {
        "name": "i",
        "type": "int128"
      },
      {
        "name": "j",
        "type": "int128"
      },
      {
        "name": "_dx",
        "type": "uint256"
      },
      {
        "name": "_min_dy",
        "type": "uint256"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ]
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "exchange",
    "inputs": [
      {
        "name": "i",
        "type": "int128"
      },
      {
        "name": "j",
        "type": "int128"
      },
      {
        "name": "_dx",
        "type": "uint256"
      },
      {
        "name": "_min_dy",
        "type": "uint256"
      },
      {
        "name": "_receiver",
        "type": "address"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ]
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "remove_liquidity",
    "inputs": [
      {
        "name": "_burn_amount",
        "type": "uint256"
      },
      {
        "name": "_min_amounts",
        "type": "uint256[2]"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256[2]"
      }
    ]
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "remove_liquidity",
    "inputs": [
      {
        "name": "_burn_amount",
        "type": "uint256"
      },
      {
        "name": "_min_amounts",
        "type": "uint256[2]"
      },
      {
        "name": "_receiver",
        "type": "address"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256[2]"
      }
    ]
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "remove_liquidity_imbalance",
    "inputs": [
      {
        "name": "_amounts",
        "type": "uint256[2]"
      },
      {
        "name": "_max_burn_amount",
        "type": "uint256"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ]
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "remove_liquidity_imbalance",
    "inputs": [
      {
        "name": "_amounts",
        "type": "uint256[2]"
      },
      {
        "name": "_max_burn_amount",
        "type": "uint256"
      },
      {
        "name": "_receiver",
        "type": "address"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ]
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "calc_withdraw_one_coin",
    "inputs": [
      {
        "name": "_burn_amount",
        "type": "uint256"
      },
      {
        "name": "i",
        "type": "int128"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 1130
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "remove_liquidity_one_coin",
    "inputs": [
      {
        "name": "_burn_amount",
        "type": "uint256"
      },
      {
        "name": "i",
        "type": "int128"
      },
      {
        "name": "_min_received",
        "type": "uint256"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ]
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "remove_liquidity_one_coin",
    "inputs": [
      {
        "name": "_burn_amount",
        "type": "uint256"
      },
      {
        "name": "i",
        "type": "int128"
      },
      {
        "name": "_min_received",
        "type": "uint256"
      },
      {
        "name": "_receiver",
        "type": "address"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ]
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "ramp_A",
    "inputs": [
      {
        "name": "_future_A",
        "type": "uint256"
      },
      {
        "name": "_future_time",
        "type": "uint256"
      }
    ],
    "outputs": [],
    "gas": 162161
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "stop_ramp_A",
    "inputs": [],
    "outputs": [],
    "gas": 157625
  },
  {
    "stateMutability": "nonpayable",
    "type": "function",
    "name": "withdraw_admin_fees",
    "inputs": [],
    "outputs": [],
    "gas": 68306
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "coins",
    "inputs": [
      {
        "name": "arg0",
        "type": "uint256"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "address"
      }
    ],
    "gas": 3093
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "admin_balances",
    "inputs": [
      {
        "name": "arg0",
        "type": "uint256"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 3123
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "fee",
    "inputs": [],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 3108
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "initial_A",
    "inputs": [],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 3138
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "future_A",
    "inputs": [],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 3168
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "initial_A_time",
    "inputs": [],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 3198
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "future_A_time",
    "inputs": [],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 3228
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "name",
    "inputs": [],
    "outputs": [
      {
        "name": "",
        "type": "string"
      }
    ],
    "gas": 13488
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "symbol",
    "inputs": [],
    "outputs": [
      {
        "name": "",
        "type": "string"
      }
    ],
    "gas": 11241
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "balanceOf",
    "inputs": [
      {
        "name": "arg0",
        "type": "address"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 3533
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "allowance",
    "inputs": [
      {
        "name": "arg0",
        "type": "address"
      },
      {
        "name": "arg1",
        "type": "address"
      }
    ],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 3778
  },
  {
    "stateMutability": "view",
    "type": "function",
    "name": "totalSupply",
    "inputs": [],
    "outputs": [
      {
        "name": "",
        "type": "uint256"
      }
    ],
    "gas": 3378
  }
]
''')


import logging

from defi_services.constants.chain_constant import Chain
from defi_services.constants.entities.dex_constant import Dex
from defi_services.jobs.queriers.state_querier import StateQuerier
from defi_services.services.dex.dex_info.sushiswap_info import SUSHISWAP_V3_ETH_INFO
from defi_services.services.dex.uniswap_v3_service import UniswapV3Services

logger = logging.getLogger("SushiSwap V3 State Service")


class SushiSwapV3Info:
    mapping = {
        Chain.ethereum: SUSHISWAP_V3_ETH_INFO
    }


class SushiSwapV3Services(UniswapV3Services):
    def __init__(self, state_service: StateQuerier, chain_id: str = '0x1'):
        super().__init__(state_service, chain_id)
        self.chain_id = chain_id
        self.state_service = state_service
        self.pool_info = SushiSwapV3Info.mapping.get(chain_id)
        self.web3 = self.state_service.get_w3()
        if self.pool_info is not None:
            self.factory_abi = self.pool_info.get('factory_abi')
            self.nft_token_manager_addr = self.pool_info.get('NFT_manager_address')
            self.nft_token_manager_abi = self.pool_info.get('NFT_manager_abi')
            self.factory_addr = self.pool_info.get('factory_address')

    def get_service_info(self):
        info = {
            Dex.sushi_v3: {
                "chain_id": self.chain_id,
                "type": "dex",
                "protocol_info": self.pool_info
            }
        }
        return info

    def get_all_supported_lp_token(self, limit: int = 100, supplied_data: dict = None):
        rpc_calls = {}
        top_token = supplied_data['token_info']
        length = min(len(top_token), limit)
        for idx0 in range(1, length):
            token0 = top_token[idx0]
            for idx1 in range(idx0 + 1, length):
                token1 = top_token[idx1]
                for fee in [100, 500, 3000, 10000]:
                    query_id = f'getPool_{self.factory_addr}_{[token0, token1, fee]}_latest'.lower()
                    rpc_calls[query_id] = self.state_service.get_function_info(
                        self.factory_addr, self.factory_abi, fn_name="getPool", fn_paras=[token0, token1, fee]
                    )
        return rpc_calls

    def decode_all_supported_lp_token(self, limit: int = 100, decoded_data: dict = None, supplied_data: dict = None):
        result = {}
        top_token = supplied_data['token_info']
        length = min(len(top_token), limit)
        for idx0 in range(1, length):
            token0 = top_token[idx0]
            for idx1 in range(idx0 + 1, length):
                token1 = top_token[idx1]
                for fee in [100, 500, 3000, 10000]:
                    query_id = f'getPool_{self.factory_addr}_{[token0, token1, fee]}_latest'.lower()
                    pool_address = decoded_data.get(query_id)
                    if pool_address != '0x0000000000000000000000000000000000000000':
                        result[pool_address] = {
                            'token0': token0,
                            'token1': token1,
                            'fee': fee
                        }

        return result



import logging

from defi_services.constants.cosmos_decimals_constant import Denoms, MulticallContract
from defi_services.constants.network_constants import Chains
from defi_services.services.cosmos_token_services import CosmosTokenServices

logger = logging.getLogger("CosmosStateProcessor")

class CosmosStateProcessor:
    def __init__(self, lcd: str, rest_uri: str = None, chain_id: str = Chains.oraichain):
        self.chain_id = chain_id
        self.lcd = lcd
        self.rest_uri = rest_uri

    def get_token_balance(self, address, tokens):
        cosmos = CosmosTokenServices(self.lcd, self.rest_uri, self.chain_id)
        if self.chain_id in MulticallContract.mapping:
            data = cosmos.query_balances(address, tokens)
        else:
            result = cosmos.query_coin_balances(address=address)
            data = {}
            for item in result:
                denom = item.get('denom', "").lower()
                amount = int(item.get('amount', None))
                decimal = Denoms.all.get(denom, {}).get("decimal", 0)
                data[denom] = amount / 10 ** decimal

        return data

    def run(self, address: str, queries: list, *args, **kwargs):
        result = []
        tokens = [query.get('entity_id').lower() for query in queries if query.get('query_type') == 'token_balance']
        token_balances = self.get_token_balance(address, tokens)
        for query in queries:
            query_id = query.get("query_id")
            entity_id = query.get("entity_id")
            query_type = query.get("query_type")
            if query_type != "token_balance":
                continue

            result.append(
                {"query_id": query_id,
                 "query_type": query_type,
                 "entity_id": entity_id,
                 "token_balance": token_balances.get(entity_id, 0)
                 })
        return result
